=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2007-11-10 13:16:12 +0000
+++ bzrlib/osutils.py	2007-11-12 21:35:15 +0000
@@ -234,10 +234,17 @@
 
     success = False
     try:
-        # This may throw an exception, in which case success will
-        # not be set.
-        rename_func(old, new)
-        success = True
+        try:
+            # This may throw an exception, in which case success will
+            # not be set.
+            rename_func(old, new)
+            success = True
+        except (IOError, OSError), e:
+            # case insensitive filesystem may be?
+            if (not file_existed
+                or e.errno not in (None, errno.ENOENT, errno.ENOTDIR)
+                or old.lower() != new.lower()):
+                raise
     finally:
         if file_existed:
             # If the file used to exist, rename it back into place
@@ -340,10 +347,6 @@
     On win32, if new exists, it must be moved out of the way first,
     and then deleted. 
     """
-    # to change case on win32 you don't need fancy rename
-    if old.lower() == new.lower():
-        os.rename(old,new)
-        return
     try:
         fancy_rename(old, new, rename_func=os.rename, unlink_func=os.unlink)
     except OSError, e:

=== modified file 'bzrlib/tests/blackbox/test_mv.py'
--- bzrlib/tests/blackbox/test_mv.py	2007-11-10 13:16:12 +0000
+++ bzrlib/tests/blackbox/test_mv.py	2007-11-12 21:35:15 +0000
@@ -17,7 +17,6 @@
 """Test for 'bzr mv'"""
 
 import os
-import sys
 
 from bzrlib import (
     osutils,
@@ -27,7 +26,6 @@
 from bzrlib.tests import (
     SymlinkFeature,
     TestCaseWithTransport,
-    TestSkipped,
     )
 
 
@@ -126,14 +124,14 @@
         os.chdir('..')
         self.assertMoved('sub1/sub2/hello.txt','sub1/hello.txt')
 
-    def test_mv_win32_change_case(self):
-        if sys.platform != 'win32':
-            raise TestSkipped('Test require case insensitive filesystem')
+    def test_mv_change_case(self):
+        # test for bug #77740 (mv unable change filename case on Windows)
         tree = self.make_branch_and_tree('.')
         self.build_tree(['test.txt'])
         tree.add(['test.txt'])
         self.run_bzr('mv test.txt Test.txt')
-        # we can't use failUnlessExists so try to check shape of the tree
+        # we can't use failUnlessExists on case-insensitive filesystem
+        # so try to check shape of the tree
         shape = sorted(os.listdir(u'.'))
         self.assertEqual(['.bzr', 'Test.txt'], shape)
         self.assertInWorkingTree('Test.txt')

=== modified file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	2007-11-10 13:16:12 +0000
+++ bzrlib/tests/test_osutils.py	2007-11-12 21:35:15 +0000
@@ -93,10 +93,13 @@
 
     # TODO: test fancy_rename using a MemoryTransport
 
-    def test_rename_win32_change_case(self):
+    def test_rename_change_case(self):
+        # on Windows we should be able to change filename case by rename
         self.build_tree(['a', 'b/'])
         osutils.rename('a', 'A')
         osutils.rename('b', 'B')
+        # we can't use failUnlessExists on case-insensitive filesystem
+        # so try to check shape of the tree
         shape = sorted(os.listdir('.'))
         self.assertEquals(['A', 'B'], shape)
 

